@@ -92,7 +92,7 @@ void QQuickBoundaryReturnJob::updateState(QAbstractAnimationJob::State newState,
 
 /*!
     \qmltype BoundaryRule
-//!    \instantiates QQuickBoundaryRule
+//!    \nativetype QQuickBoundaryRule
     \inqmlmodule Qt.labs.animation
     \ingroup qtquick-transitions-animations
     \ingroup qtquick-interceptors
@@ -111,7 +111,7 @@ void QQuickBoundaryReturnJob::updateState(QAbstractAnimationJob::State newState,
     Note that a property cannot have more than one assigned BoundaryRule.
 
     \sa {Animation and Transitions in Qt Quick}, {Qt Quick Examples - Animation#Behaviors}{Behavior
-example}, {Qt QML}, {Qt Quick Examples - Pointer Handlers}
+example}, {Qt Qml}, {Qt Quick Examples - Pointer Handlers}
 */
 
 QQuickBoundaryRule::QQuickBoundaryRule(QObject *parent)
@@ -152,7 +152,7 @@ void QQuickBoundaryRule::setEnabled(bool enabled)
 }
 
 /*!
-    \qmlproperty qreal Qt.labs.animation::BoundaryRule::minimum
+    \qmlproperty real Qt.labs.animation::BoundaryRule::minimum
 
     This property holds the smallest unconstrained value that the property is
     allowed to have.  If the property is set to a smaller value, it will be
@@ -176,7 +176,7 @@ void QQuickBoundaryRule::setMinimum(qreal minimum)
 }
 
 /*!
-    \qmlproperty qreal Qt.labs.animation::BoundaryRule::minimumOvershoot
+    \qmlproperty real Qt.labs.animation::BoundaryRule::minimumOvershoot
 
     This property holds the amount that the property is allowed to be
     less than \l minimum.  Whenever the value is less than \l minimum
@@ -202,7 +202,7 @@ void QQuickBoundaryRule::setMinimumOvershoot(qreal minimumOvershoot)
 }
 
 /*!
-    \qmlproperty qreal Qt.labs.animation::BoundaryRule::maximum
+    \qmlproperty real Qt.labs.animation::BoundaryRule::maximum
 
     This property holds the largest unconstrained value that the property is
     allowed to have.  If the property is set to a larger value, it will be
@@ -226,7 +226,7 @@ void QQuickBoundaryRule::setMaximum(qreal maximum)
 }
 
 /*!
-    \qmlproperty qreal Qt.labs.animation::BoundaryRule::maximumOvershoot
+    \qmlproperty real Qt.labs.animation::BoundaryRule::maximumOvershoot
 
     This property holds the amount that the property is allowed to be
     more than \l maximum.  Whenever the value is greater than \l maximum
@@ -252,7 +252,7 @@ void QQuickBoundaryRule::setMaximumOvershoot(qreal maximumOvershoot)
 }
 
 /*!
-    \qmlproperty qreal Qt.labs.animation::BoundaryRule::overshootScale
+    \qmlproperty real Qt.labs.animation::BoundaryRule::overshootScale
 
     This property holds the amount by which the \l easing is scaled during the
     overshoot condition. For example if an Item is restricted from moving more
@@ -284,7 +284,7 @@ void QQuickBoundaryRule::setOvershootScale(qreal overshootScale)
 }
 
 /*!
-    \qmlproperty qreal Qt.labs.animation::BoundaryRule::currentOvershoot
+    \qmlproperty real Qt.labs.animation::BoundaryRule::currentOvershoot
 
     This property holds the amount by which the most recently set value of the
     intercepted property exceeds \l maximum or is less than \l minimum.
@@ -300,7 +300,7 @@ qreal QQuickBoundaryRule::currentOvershoot() const
 }
 
 /*!
-    \qmlproperty qreal Qt.labs.animation::BoundaryRule::peakOvershoot
+    \qmlproperty real Qt.labs.animation::BoundaryRule::peakOvershoot
 
     This property holds the most-positive or most-negative value of
     \l currentOvershoot that has been seen, until \l returnToBounds() is called.
