@@ -3,7 +3,6 @@
 
 #include "qv4arrayiterator_p.h"
 #include "qv4urlobject_p.h"
-#include "qv4stringobject_p.h"
 
 #include <QtCore/QUrl>
 
@@ -19,9 +18,9 @@ DEFINE_OBJECT_VTABLE(UrlSearchParamsObject);
 DEFINE_OBJECT_VTABLE(UrlSearchParamsCtor);
 
 
-void Heap::UrlCtor::init(QV4::ExecutionContext *scope)
+void Heap::UrlCtor::init(QV4::ExecutionEngine *engine)
 {
-    Heap::FunctionObject::init(scope, QLatin1String("URL"));
+    Heap::FunctionObject::init(engine, QLatin1String("URL"));
 }
 
 void UrlPrototype::init(ExecutionEngine *engine, Object *ctor)
@@ -125,14 +124,14 @@ void UrlObject::setUrl(const QUrl &url)
 {
     d()->hash.set(engine(), engine()->newString(url.fragment()));
     d()->hostname.set(engine(), engine()->newString(url.host()));
-    d()->href.set(engine(), engine()->newString(url.toString()));
+    d()->href.set(engine(), engine()->newString(url.toString(QUrl::ComponentFormattingOptions(QUrl::ComponentFormattingOption::FullyEncoded))));
     d()->password.set(engine(), engine()->newString(url.password()));
     d()->pathname.set(engine(), engine()->newString(url.path()));
     d()->port.set(engine(),
                   engine()->newString(url.port() == -1 ? QLatin1String("")
                                                        : QString::number(url.port())));
     d()->protocol.set(engine(), engine()->newString(url.scheme() + QLatin1Char(':')));
-    d()->search.set(engine(), engine()->newString(url.query()));
+    d()->search.set(engine(), engine()->newString(url.query(QUrl::ComponentFormattingOptions(QUrl::ComponentFormattingOption::FullyEncoded))));
     d()->username.set(engine(), engine()->newString(url.userName()));
 
     updateOrigin();
@@ -243,6 +242,18 @@ bool UrlObject::setUsername(QString username)
     return true;
 }
 
+QString UrlObject::search() const
+{
+    auto url = QUrl(href());
+    if (auto url = QUrl(href()); !url.hasQuery() || url.query().isEmpty())
+        return QLatin1String("");
+
+    constexpr auto options = QUrl::ComponentFormattingOption::EncodeSpaces
+            | QUrl::ComponentFormattingOption::EncodeUnicode
+            | QUrl::ComponentFormattingOption::EncodeReserved;
+    return u'?' + url.query(options);
+}
+
 QUrl UrlObject::toQUrl() const
 {
     return QUrl(href());
@@ -671,6 +682,7 @@ ReturnedValue UrlPrototype::method_getSearchParams(const FunctionObject *b, cons
 
     Scoped<UrlSearchParamsObject> usp(scope, v4->newUrlSearchParamsObject());
 
+    usp->setUrlObject(thisObject->as<UrlObject>());
     usp->initializeParams(r->search());
 
     return usp->asReturnedValue();
@@ -738,9 +750,9 @@ ReturnedValue UrlCtor::virtualCallAsConstructor(const FunctionObject *that, cons
 }
 
 
-void Heap::UrlSearchParamsCtor::init(QV4::ExecutionContext *scope)
+void Heap::UrlSearchParamsCtor::init(QV4::ExecutionEngine *engine)
 {
-    Heap::FunctionObject::init(scope, QLatin1String("URLSearchParams"));
+    Heap::FunctionObject::init(engine, QLatin1String("URLSearchParams"));
 }
 
 void UrlSearchParamsPrototype::init(ExecutionEngine *engine, Object *ctor)
@@ -794,7 +806,8 @@ ReturnedValue UrlSearchParamsCtor::virtualCallAsConstructor(const FunctionObject
         uint len = argArray->getLength();
 
         for (uint i = 0; i < len; i++) {
-            QV4::Value pair = argArray->get(i);
+            // safe to user fromReturnedValue: Normal array, which will ensure marking
+            QV4::Value pair = Value::fromReturnedValue(argArray->get(i));
             auto *pairArrayObject = pair.as<ArrayObject>();
 
             if (pairArrayObject == nullptr) {
@@ -883,11 +896,14 @@ void UrlSearchParamsObject::initializeParams(ScopedArrayObject& params)
 
     for (uint i = 0; i < len; i++)
     {
-        QV4::Value pair = params->get(i);
+        // fromReturnedValue is safe; everything is reachable via params
+        // so the gc won't collect it; we control params, so there can't be
+        // any weird proxy magic
+        QV4::Value pair = Value::fromReturnedValue(params->get(i));
         auto *pairArrayObject = pair.as<ArrayObject>();
 
-        QV4::Value key = pairArrayObject->get(uint(0));
-        QV4::Value value = pairArrayObject->get(uint(1));
+        QV4::Value key = Value::fromReturnedValue(pairArrayObject->get(uint(0)));
+        QV4::Value value = Value::fromReturnedValue(pairArrayObject->get(uint(1)));
 
         scopedKeys->put(i, key);
         scopedValues->put(i, value);
@@ -961,6 +977,11 @@ void UrlSearchParamsObject::setParams(QList<QStringList> params)
     d()->values.set(engine(), values);
 }
 
+void UrlSearchParamsObject::setUrlObject(const UrlObject *url)
+{
+    d()->url.set(engine(), url->d());
+}
+
 void UrlSearchParamsObject::append(Heap::String *name, Heap::String *value)
 {
     Scope scope(engine());
@@ -997,11 +1018,11 @@ QList<QStringList> UrlSearchParamsObject::params() const
     uint len = scopedArray->getLength();
 
     for (uint i = 0; i < len; i++) {
-        QV4::Value pair = scopedArray->get(i);
+        QV4::Value pair = Value::fromReturnedValue(scopedArray->get(i));
         auto *pairArrayObject = pair.as<ArrayObject>();
 
-        QV4::Value key = pairArrayObject->get(uint(0));
-        QV4::Value value = pairArrayObject->get(uint(1));
+        QV4::Value key = Value::fromReturnedValue(pairArrayObject->get(uint(0)));
+        QV4::Value value = Value::fromReturnedValue(pairArrayObject->get(uint(1)));
 
         result << QStringList { key.toQString(), value.toQString() };
     }
@@ -1009,6 +1030,25 @@ QList<QStringList> UrlSearchParamsObject::params() const
     return result;
 }
 
+Heap::UrlObject *UrlSearchParamsObject::urlObject() const
+{
+    return d()->url.get();
+}
+
+QString UrlSearchParamsObject::searchString() const
+{
+    QString search = QLatin1String("");
+    auto params = this->params();
+    auto len = params.size();
+    for (int i = 0; i < len; ++i) {
+        const QStringList &param = params[i];
+        search += param[0] + QLatin1Char('=') + param[1];
+        if (i != len - 1)
+            search += QLatin1Char('&');
+    }
+    return search;
+}
+
 int UrlSearchParamsObject::length() const
 {
     auto *arrayObject = d()->params.get();
@@ -1027,10 +1067,11 @@ int UrlSearchParamsObject::indexOf(QString name, int last) const
     int len = scopedArray->getLength();
 
     for (int i = last + 1; i < len; i++) {
-        QV4::Value pair = scopedArray->get(i);
+        // fromReturnedValue is safe, scopedArray is a normal array and takes care of marking
+        QV4::Value pair = Value::fromReturnedValue(scopedArray->get(i));
         auto *pairArrayObject = pair.as<ArrayObject>();
 
-        QV4::Value key = pairArrayObject->get(uint(0));
+        QV4::Value key = Value::fromReturnedValue(pairArrayObject->get(uint(0)));
 
         if (key.toQString() == name)
             return i;
@@ -1048,10 +1089,10 @@ QString UrlSearchParamsObject::stringAt(int index, int pairIndex) const
     if (index >= scopedArray->getLength())
         return {};
 
-    QV4::Value pair = scopedArray->get(index);
+    QV4::Value pair = Value::fromReturnedValue(scopedArray->get(index));
     auto *pairArrayObject = pair.as<ArrayObject>();
 
-    QV4::Value value = pairArrayObject->get(pairIndex);
+    QV4::Value value = Value::fromReturnedValue(pairArrayObject->get(pairIndex));
 
     return value.toQString();
 }
@@ -1065,10 +1106,10 @@ QV4::Heap::String * UrlSearchParamsObject::stringAtRaw(int index, int pairIndex)
     if (index >= scopedArray->getLength())
         return nullptr;
 
-    QV4::Value pair = scopedArray->get(index);
+    QV4::Value pair = Value::fromReturnedValue(scopedArray->get(index));
     auto *pairArrayObject = pair.as<ArrayObject>();
 
-    QV4::Value value = pairArrayObject->get(pairIndex);
+    QV4::Value value = Value::fromReturnedValue(pairArrayObject->get(pairIndex));
 
     return value.as<String>()->d();
 }
@@ -1252,12 +1293,7 @@ ReturnedValue UrlSearchParamsPrototype::method_delete(const FunctionObject *b, c
         return Encode::undefined();
 
     QList<QStringList> params = o->params();
-
-    auto to_remove = std::remove_if(params.begin(), params.end(), [&name](QStringList pair) {
-                                                                      return pair[0] == name;
-                                                                  });
-
-    params.erase(to_remove, params.end());
+    params.removeIf([&name](const auto &pair) { return pair.at(0) == name; });
 
     o->setParams(params);
 
@@ -1336,6 +1372,10 @@ ReturnedValue UrlSearchParamsPrototype::method_set(const FunctionObject *b, cons
 
     o->setParams(params);
 
+    Scoped<UrlObject> scopedUrlObject(scope, o->d()->url.get());
+    if (scopedUrlObject)
+        scopedUrlObject->setSearch(o->searchString());
+
     return Encode::undefined();
 }
 
