@@ -2,12 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.2-2-19
+es5id: 15.2.3.2-2-8
 description: >
     Object.getPrototypeOf returns the [[Prototype]] of its parameter
-    (Object object)
+    (Math)
 ---*/
 
-var obj = {};
-
-assert.sameValue(Object.getPrototypeOf(obj), Object.prototype, 'Object.getPrototypeOf(obj)');
+assert.sameValue(Object.getPrototypeOf(Math), Object.prototype, 'Object.getPrototypeOf(Math)');
