@@ -2,12 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.11-4-9
-description: >
-    Object.isSealed returns false for all built-in objects
-    (String.prototype)
+es5id: 15.2.3.11-4-8
+description: Object.isSealed returns false for all built-in objects (String)
 ---*/
 
-var b = Object.isSealed(String.prototype);
+var b = Object.isSealed(String);
 
 assert.sameValue(b, false, 'b');
