@@ -2,11 +2,11 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 11.13.2-6-14-s
+es5id: 11.13.2-6-12-s
 description: >
     Strict Mode - SyntaxError is thrown if the identifier arguments
     appear as the LeftHandSideExpression of a Compound Assignment
-    operator(%=)
+    operator(*=)
 flags: [onlyStrict]
 ---*/
 
@@ -14,7 +14,7 @@ function testcase() {
         var err = null;
         var blah = arguments;
         try {
-            eval("arguments %= 20;");
+            eval("arguments *= 20;");
         } catch (e) {
             err = e;
         }
