@@ -1,13 +1,17 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-#include <qtest.h>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+
+#include "WithBindableProperties.h"
+
+#include <private/qmlutils_p.h>
+#include <private/qqmlbind_p.h>
+#include <private/qqmlcomponentattached_p.h>
+#include <private/qquickrectangle_p.h>
+
+#include <QtTest/qtest.h>
+
 #include <QtQml/qqmlengine.h>
 #include <QtQml/qqmlcomponent.h>
-#include <QtQml/private/qqmlbind_p.h>
-#include <QtQml/private/qqmlcomponentattached_p.h>
-#include <QtQuick/private/qquickrectangle_p.h>
-#include <QtQuickTestUtils/private/qmlutils_p.h>
-#include "WithBindableProperties.h"
 
 class tst_qqmlbinding : public QQmlDataTest
 {
@@ -25,6 +29,7 @@ private slots:
     void restoreBindingJSValue();
     void restoreBindingWithLoop();
     void restoreBindingWithoutCrash();
+    void restoreBindingWhenDestroyed();
     void deletedObject();
     void warningOnUnknownProperty();
     void warningOnReadOnlyProperty();
@@ -38,6 +43,9 @@ private slots:
     void intOverflow();
     void generalizedGroupedProperties();
     void localSignalHandler();
+    void whenEvaluatedEarlyEnough();
+    void propertiesAttachedToBindingItself();
+    void toggleEnableProperlyRemembersValues();
 
 private:
     QQmlEngine engine;
@@ -46,6 +54,9 @@ private:
 tst_qqmlbinding::tst_qqmlbinding()
     : QQmlDataTest(QT_QMLTEST_DATADIR)
 {
+#ifdef QML_DISABLE_INTERNAL_DEFERRED_PROPERTIES
+    qputenv("QML_DISABLE_INTERNAL_DEFERRED_PROPERTIES", "1");
+#endif
 }
 
 void tst_qqmlbinding::binding()
@@ -148,8 +159,11 @@ void tst_qqmlbinding::restoreBindingValue()
 {
     QQmlEngine engine;
     QQmlComponent c(&engine, testFileUrl("restoreBinding2.qml"));
-    QScopedPointer<QQuickRectangle> rect(qobject_cast<QQuickRectangle*>(c.create()));
-    QVERIFY(!rect.isNull());
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QQuickRectangle *rect = qobject_cast<QQuickRectangle*>(o.data());
+    QVERIFY(rect);
 
     auto myItem = qobject_cast<QQuickRectangle*>(rect->findChild<QQuickRectangle*>("myItem"));
     QVERIFY(myItem != nullptr);
@@ -172,8 +186,11 @@ void tst_qqmlbinding::restoreBindingVarValue()
 {
     QQmlEngine engine;
     QQmlComponent c(&engine, testFileUrl("restoreBinding3.qml"));
-    QScopedPointer<QQuickRectangle> rect(qobject_cast<QQuickRectangle*>(c.create()));
-    QVERIFY(!rect.isNull());
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QQuickRectangle *rect = qobject_cast<QQuickRectangle*>(o.data());
+    QVERIFY(rect);
 
     auto myItem = qobject_cast<QQuickRectangle*>(rect->findChild<QQuickRectangle*>("myItem"));
     QVERIFY(myItem != nullptr);
@@ -196,8 +213,11 @@ void tst_qqmlbinding::restoreBindingJSValue()
 {
     QQmlEngine engine;
     QQmlComponent c(&engine, testFileUrl("restoreBinding4.qml"));
-    QScopedPointer<QQuickRectangle> rect(qobject_cast<QQuickRectangle*>(c.create()));
-    QVERIFY(!rect.isNull());
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QQuickRectangle *rect = qobject_cast<QQuickRectangle*>(o.data());
+    QVERIFY(rect);
 
     auto myItem = qobject_cast<QQuickRectangle*>(rect->findChild<QQuickRectangle*>("myItem"));
     QVERIFY(myItem != nullptr);
@@ -287,6 +307,25 @@ void tst_qqmlbinding::restoreBindingWithoutCrash()
     QCOMPARE(myItem->x(), qreal(100-49));
 }
 
+void tst_qqmlbinding::restoreBindingWhenDestroyed()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("bindingRestoredWhenDestroyed.qml"));
+    QScopedPointer<QObject> root {c.create()};
+    QVERIFY2(root, qUtf8Printable(c.errorString()));
+    QCOMPARE(root->property("text").toString(), u"original");
+    QCOMPARE(root->property("i").toInt(), 42);
+
+    root->setProperty("toggle", true);
+    // QTRY_COMPARE as loader is async
+    QTRY_COMPARE(root->property("text").toString(), u"changed");
+    QCOMPARE(root->property("i").toInt(), 100);
+
+    root->setProperty("toggle", false);
+    QTRY_COMPARE(root->property("text").toString(), u"original");
+    QCOMPARE(root->property("i").toInt(), 100);
+}
+
 //QTBUG-20692
 void tst_qqmlbinding::deletedObject()
 {
@@ -356,10 +395,6 @@ void tst_qqmlbinding::disabledOnReadonlyProperty()
 
 void tst_qqmlbinding::delayed()
 {
-#ifdef Q_OS_ANDROID
-    QSKIP("This test crashes on Android. QTBUG-103310");
-#endif
-
     QQmlEngine engine;
     QQmlComponent c(&engine, testFileUrl("delayed.qml"));
     QScopedPointer<QQuickItem> item {qobject_cast<QQuickItem*>(c.create())};
@@ -446,9 +481,15 @@ void tst_qqmlbinding::bindingOverwriting()
     QQmlComponent c(&engine, testFileUrl("bindingOverwriting.qml"));
     QScopedPointer<QQuickItem> item {qobject_cast<QQuickItem*>(c.create())};
     QVERIFY(item);
+    QCOMPARE(messageHandler.messages().size(), 2);
+
+    QQmlComponent c2(&engine, testFileUrl("bindingOverwriting2.qml"));
+    QScopedPointer<QObject> o(c2.create());
+    QVERIFY(o);
+    QTRY_COMPARE(o->property("i").toInt(), 123);
+    QCOMPARE(messageHandler.messages().size(), 3);
 
     QLoggingCategory::setFilterRules(QString());
-    QCOMPARE(messageHandler.messages().size(), 2);
 }
 
 void tst_qqmlbinding::bindToQmlComponent()
@@ -584,6 +625,55 @@ void tst_qqmlbinding::localSignalHandler()
     QCOMPARE(o->property("output").toString(), QStringLiteral("abc"));
 }
 
+void tst_qqmlbinding::whenEvaluatedEarlyEnough()
+{
+    QQmlEngine e;
+    QQmlComponent c(&e, testFileUrl("whenEvaluatedEarlyEnough.qml"));
+    QTest::failOnWarning(QRegularExpression(".*"));
+    std::unique_ptr<QObject> root { c.create() };
+    root->setProperty("toggle", false); // should not cause warnings
+    // until "when" is actually true
+    QTest::ignoreMessage(QtMsgType::QtWarningMsg,
+                         QRegularExpression(".*QML Binding: Property 'i' does not exist on Item.*"));
+    root->setProperty("forceEnable", true);
+}
+
+void tst_qqmlbinding::propertiesAttachedToBindingItself()
+{
+    QQmlEngine e;
+    QQmlComponent c(&e, testFileUrl("propertiesAttachedToBindingItself.qml"));
+    QTest::failOnWarning(QRegularExpression(".*"));
+    std::unique_ptr<QObject> root { c.create() };
+    QVERIFY2(root, qPrintable(c.errorString()));
+    // 0 => everything broken; 1 => normal attached properties broken;
+    // 2 => Component.onCompleted broken, 3 => everything works
+    QTRY_COMPARE(root->property("check").toInt(), 3);
+}
+
+void tst_qqmlbinding::toggleEnableProperlyRemembersValues()
+{
+    QQmlEngine e;
+    QQmlComponent c(&e, testFileUrl("toggleEnableProperlyRemembersValues.qml"));
+    std::unique_ptr<QObject> root { c.create() };
+    QVERIFY2(root, qPrintable(c.errorString()));
+    for (int i = 0; i < 3; ++i) {
+        {
+            QJSManagedValue arr(root->property("arr"), &e);
+            QJSManagedValue func(root->property("func"), &e);
+            QCOMPARE(arr.property("length").toInt(), 2);
+            QCOMPARE(func.call().toInt(), 1);
+        }
+        root->setProperty("enabled", true);
+        {
+            QJSManagedValue arr(root->property("arr"), &e);
+            QJSManagedValue func(root->property("func"), &e);
+            QCOMPARE(arr.property("length").toInt(), 3);
+            QCOMPARE(func.call().toInt(), 2);
+        }
+        root->setProperty("enabled", false);
+    }
+}
+
 QTEST_MAIN(tst_qqmlbinding)
 
 #include "tst_qqmlbinding.moc"
